@@ -3,8 +3,13 @@ from unittest import TestCase
 
 import numpy as np
 
-from voxypy.models.models import Entity, validate_and_pad_texture
+from voxypy.models import Entity, validate_and_pad_palette, Color
 
+HOWDY_FILE = os.path.sep.join(['test', 'howdy.vox'])
+RED_FILE = os.path.sep.join(["test", "red.png"])
+SINGLE_FILE = os.path.sep.join(["test", "single.vox"])
+SINGLE_2_FILE = os.path.sep.join(["test", "single_2.vox"])
+TINY_FILE = os.path.sep.join(['test', 'tiny.vox'])
 
 def matrices_are_equal(m1, m2):
     return all([m1[x][y] == m2[x][y] for x in range(len(m1)) for y in range(len(m1[0]))])
@@ -121,15 +126,15 @@ class TestEntity(TestCase):
         self.assertTrue(matrices_are_equal(layer, actual), "Expected %s\n\nbut received %s" % (layer, actual))
 
     def test_read_from_file(self):
-        filename = os.path.sep.join(['test', 'howdy.vox'])
+        filename = HOWDY_FILE
         try:
             entity = Entity().from_file(filename)
         except Exception as exc:
             self.fail(f"Failed to read from file {filename} :: {exc}")
 
     def test_read_write(self):
-        in_filename = os.path.sep.join([os.getcwd(), 'test', 'howdy.vox'])
-        out_filename = os.path.sep.join([os.getcwd(), 'test', 'howdy2.vox'])
+        in_filename = SINGLE_FILE
+        out_filename = SINGLE_2_FILE
         entity = Entity()
         entity2 = Entity()
         try:
@@ -151,13 +156,13 @@ class TestEntity(TestCase):
         self.assertEqual(entity, entity2, f"Entities should be the same before and after writing.\n"
                                           f"Entity 1:\n{entity}\n{print_slices(entity.get_dense())}\n"
                                           f"Entity 2:\n{entity2}\n{print_slices(entity2.get_dense())}")
+
         try:
             os.remove(out_filename)
         except Exception as exc:
             print(f"Failed to clean up after unit test. File: {out_filename}")
 
     def test_init_vs_file(self):
-        tiny_file = os.path.sep.join([os.getcwd(), 'test', 'tiny.vox'])
         tiny_mem = np.zeros((1, 2, 3), dtype=int)
         i = 1
 
@@ -166,80 +171,83 @@ class TestEntity(TestCase):
                 for x in range(1):
                     tiny_mem[x, y, z] = i
                     i += 1
-        tiny_disk = Entity().from_file(tiny_file).get_dense()
+        tiny_disk = Entity().from_file(TINY_FILE).get_dense()
         self.assertTrue(np.array_equal(tiny_mem, tiny_disk), f"\nMemory:\n{print_layers(tiny_mem)}"
                                                              f"\nDisk:\n{print_layers(tiny_disk)}")
 
-    def test_init_texture_and_tex_file(self):
-        try:
-            texture = [(i, i, i) for i in range(255)]
-            entity = Entity(texture=texture, texture_file="blah")
-            self.fail(f"Two versions of texture should have failed.")
-        except ValueError:
-            pass
+    def test_read_palette_from_vox_file(self):
+        entity = Entity().from_file(SINGLE_FILE)
+        expected = [Color(75, 75, 75, 255)] * 255
+        self.assertEqual(expected, entity.get_palette(padded=False))
 
-    def test_validate_pad_texture_works(self):
+    def test_read_palette_after_import(self):
+        entity = Entity().from_file(SINGLE_FILE)
+        entity.set_palette_from_file(RED_FILE)
+        expected = [Color(255, 0, 0, 255)] * 255
+        self.assertEqual(expected, entity.get_palette(padded=False))
+
+    def test_validate_pad_palette_works(self):
         try:
-            texture = [(i, i, i) for i in range(255)]
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(255)]
+            validate_and_pad_palette(palette)
         except ValueError as exc:
             self.fail(f"Validation for working case failed: {exc}")
 
-    def test_validate_pad_texture_pads(self):
-        texture = [(i, i, i) for i in range(10)]
-        validate_and_pad_texture(texture)
-        self.assertEqual(255, len(texture), f"Function should return padded "
-                                            f"texture with length 255 but received length {len(texture)}")
+    def test_validate_pad_palette_pads(self):
+        palette = [(i, i, i, i) for i in range(10)]
+        validate_and_pad_palette(palette)
+        self.assertEqual(255, len(palette), f"Function should return padded "
+                                            f"palette with length 255 but received length {len(palette)}")
 
-    def test_validate_pad_texture_too_long(self):
+    def test_validate_pad_palette_too_long(self):
         try:
-            texture = [(i, i, i) for i in range(256)]
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(257)]
+            validate_and_pad_palette(palette)
             self.fail("Should have failed length check.")
         except ValueError:
             pass
 
-    def test_validate_pad_texture_bad_data(self):
+    def test_validate_pad_palette_bad_data(self):
         try:
-            texture = [(i, i, i) for i in range(255)]
-            texture[0] = "invalid"
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(255)]
+            palette[0] = "invalid"
+            validate_and_pad_palette(palette)
             self.fail("Should have failed bad data check.")
         except ValueError:
             pass
 
-    def test_validate_pad_texture_bad_color(self):
+    def test_validate_pad_palette_bad_color(self):
         try:
-            texture = [(i, i, i) for i in range(255)]
-            texture[0] = (1, 2, 3, 4)
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(255)]
+            palette[0] = (1, 2, 3)
+            validate_and_pad_palette(palette)
             self.fail("Should have failed color length check.")
         except ValueError:
             pass
 
-    def test_validate_pad_texture_bad_channel_data(self):
+    def test_validate_pad_palette_bad_channel_data(self):
         try:
-            texture = [(i, i, i) for i in range(255)]
-            texture[0] = ("invalid", 2, 3)
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(255)]
+            palette[0] = ("invalid", 2, 3, 255)
+            validate_and_pad_palette(palette)
             self.fail("Should have failed channel data check.")
         except ValueError:
             pass
 
-    def test_validate_pad_texture_bad_channel_low(self):
+    def test_validate_pad_palette_bad_channel_low(self):
         try:
-            texture = [(i, i, i) for i in range(255)]
-            texture[0] = (-1, 2, 3)
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(255)]
+            palette[0] = (-1, 2, 3, 255)
+            validate_and_pad_palette(palette)
             self.fail("Should have failed channel data check.")
         except ValueError:
             pass
 
-    def test_validate_pad_texture_bad_channel_high(self):
+    def test_validate_pad_palette_bad_channel_high(self):
         try:
-            texture = [(i, i, i) for i in range(255)]
-            texture[0] = (1, 2, 256)
-            validate_and_pad_texture(texture)
+            palette = [(i, i, i, i) for i in range(255)]
+            palette[0] = (1, 2, 256, 255)
+            validate_and_pad_palette(palette)
             self.fail("Should have failed channel data check.")
         except ValueError:
             pass
